package za.co.enerweb.testorient;

import com.google.common.base.Stopwatch;
import com.google.common.util.concurrent.ListeningExecutorService;
import com.google.common.util.concurrent.MoreExecutors;
import com.orientechnologies.orient.core.db.graph.OGraphDatabase;
import com.orientechnologies.orient.core.metadata.schema.OClass;
import com.orientechnologies.orient.core.metadata.schema.OType;
import com.tinkerpop.blueprints.Vertex;
import com.tinkerpop.blueprints.impls.orient.OrientGraphNoTx;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public class App {

    private static int count = 0;

    public static void main(String[] args) throws SAXException, FileNotFoundException {
        InputSource inputSource = new InputSource(new InputStreamReader(new BufferedInputStream(new FileInputStream("/tmp/Johannesburg.osm"))));

        String dburl = "remote:localhost/OSM2";
        OGraphDatabase odb = new OGraphDatabase(dburl).open("admin", "admin");
        final OrientGraphNoTx graph = new OrientGraphNoTx(odb);
        odb.setUseCustomTypes(true);
        final OClass node = odb.createVertexType("Node");
        node.createProperty("nodeid", OType.LONG);
        node.createProperty("lat", OType.DOUBLE);
        node.createProperty("lon", OType.DOUBLE);

        node.createIndex("nodeididx", OClass.INDEX_TYPE.UNIQUE, "nodeid");

        //node.createIndex("nodeididx", OClass.INDEX_TYPE.UNIQUE, "nodeid");



        //
        XMLReader xmlReader = XMLReaderFactory.createXMLReader();
        xmlReader.setContentHandler(new DefaultHandler() {
            private long wayId;
            private final List<Vertex> vertexs = new ArrayList<>();

            public void startElement(String uri, String localName, String name, org.xml.sax.Attributes attributes) throws SAXException {
                count++;


                if (name.equals("node")) {
                    node(attributes);
                } else if (name.equals("way")) {
                    way(attributes);
                } else if (name.equals("nd")) {
                    Long nodeId = Long.parseLong(attributes.getValue("ref"));
                    nd(nodeId);
                }
            }

            public void endElement(String uri, String localName, String name) throws SAXException {
                if (endElement(name)) {
                    return;
                }
            }

            private boolean endElement(String name) {
                //Wait until we get closing <way> tag , so that <nd> and <tag> are already loaded

                if (name.equals("way")) {
                    Stopwatch stopwatch = new Stopwatch().start();
                    System.out.println("connect ways " + count + " vertexs " + vertexs.size());
                    Vertex vertex = null;
                    if (vertexs.size() > 1) {
                        vertex = vertexs.get(0);
                    } else {
                        vertexs.clear();
                        return true;
                    }
                    for (int count = 1; count < vertexs.size(); count++) {
                        Vertex next = vertexs.get(count);
                        graph.addEdge(wayId, vertex, next, "connects");
                        vertex = next;
                    }
                    graph.commit();
                    vertexs.clear();
                    System.out.println("way " + stopwatch.elapsed(TimeUnit.MILLISECONDS) + "ms");
                }
                return false;
            }

            private void way(Attributes attributes) throws NumberFormatException {
                //will handle "way" and it's "nd" sub-nodes

                //for (int count = 0; count < attributes.getLength(); count++) {
                wayId = Long.parseLong(attributes.getValue("id"));
                //}
            }

            private void nd(Long nodeId) {
                Stopwatch stopwatch = new Stopwatch().start();
                Iterable<Vertex> vs = graph.query().has("nodeid", nodeId).limit(1).vertices();
                //Iterable<Vertex> vs = graph.getVertices("nodeid", nodeId);
                final Iterator<Vertex> it = vs.iterator();
                if (it.hasNext()) {
                    Vertex v = it.next();
                    vertexs.add(v);
                }
                stopwatch.stop();
                System.out.println("nd " + stopwatch.elapsed(TimeUnit.MILLISECONDS) + "ms");
            }

            private void node(Attributes attributes) {
                Stopwatch stopwatch = new Stopwatch().start();
                System.out.println("node " + count);
                wayId = -1;
                Object id = attributes.getValue("id");
                Object lat = attributes.getValue("lat");
                Object lon = attributes.getValue("lon");
                //System.out.println("id " + id + " lat " + lat + " lon " + lon);
                Vertex vNode = graph.addVertex("class:Node");
                vNode.setProperty("nodeid", id);
                vNode.setProperty("lat", lat);
                vNode.setProperty("lon", lon);

                graph.commit();
                System.out.println("node " + stopwatch.elapsed(TimeUnit.MILLISECONDS) + "ms");

                // Switch handler to parse the node element
            }
        });
        try {
            xmlReader.parse(inputSource);
        } catch (IOException ex) {
            Logger.getLogger(App.class.getName()).log(Level.SEVERE, null, ex);
        }



    }
}
